"""
*******************
Linear Mixed Models
*******************

- :class:`.FastLMM`
- :class:`.SlowLMM`

Linear mean
^^^^^^^^^^^

.. autoclass:: FastLMM
  :members:

Offset mean
^^^^^^^^^^^

.. autoclass:: SlowLMM
  :members:
"""

from ._fastlmm import FastLMM
from ._slowlmm import SlowLMM
